﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Web;
using System.Web.Caching;
using Microsoft.Practices.EnterpriseLibrary.PolicyInjection.CallHandlers;
using Microsoft.Practices.Unity.InterceptionExtension;

namespace CachingCallHandler4OutParam
{
    public class FixedCachingCallHandler : ICallHandler
    {
        // Fields
        public static readonly TimeSpan DefaultExpirationTime = new TimeSpan(0, 5, 0);
        private TimeSpan expirationTime;
        private ICacheKeyGenerator keyGenerator;
        private int order;

        // Methods
        public FixedCachingCallHandler()
            : this(DefaultExpirationTime)
        {
        }

        public FixedCachingCallHandler(TimeSpan expirationTime)
        {
            this.keyGenerator = new DefaultCacheKeyGenerator();
            this.expirationTime = expirationTime;
        }

        public FixedCachingCallHandler(TimeSpan expirationTime, int order)
        {
            this.keyGenerator = new DefaultCacheKeyGenerator();
            this.expirationTime = expirationTime;
            this.order = order;
        }

        private void AddToCache(string key, object value)
        {
            object[] objArray = new object[] { value };
            HttpRuntime.Cache.Insert(key, objArray, null, Cache.NoAbsoluteExpiration, this.expirationTime, CacheItemPriority.Normal, null);
        }

        private void AddToCache(string key, InvocationResult result)
        {
            HttpRuntime.Cache.Insert(key, result, null, Cache.NoAbsoluteExpiration, this.expirationTime, CacheItemPriority.Normal, null);
        }

        public IMethodReturn Invoke(IMethodInvocation input, GetNextHandlerDelegate getNext)
        {           
            object[] inputs = new object[input.Inputs.Count];
            for (int i = 0; i < inputs.Length; i++)
            {
                inputs[i] = input.Inputs[i];
            }
            string key = this.keyGenerator.CreateCacheKey(input.MethodBase, inputs);
            InvocationResult result = (InvocationResult)HttpRuntime.Cache.Get(key);
            if (result == null)
            {
                IMethodReturn return2 = getNext()(input, getNext);
                if (return2.Exception == null)
                {
                    this.AddToCache(key, this.GetInvocationResult(input, return2));
                }
                return return2;
            }
            var returnValue = input.CreateMethodReturn(result.ReturnValue, result.StreamlineArguments(input.Arguments));

            return returnValue;
        }

        private InvocationResult GetInvocationResult(IMethodInvocation input, IMethodReturn methodReturn)
        {
            var outParms = new List<OutputParameter>();

            for (int i = 0; i < input.Arguments.Count; i++)
            {
                ParameterInfo paramInfo = input.Arguments.GetParameterInfo(i);
                if (paramInfo.IsOut)
                {
                    OutputParameter param = new OutputParameter(input.Arguments[i], i);
                    outParms.Add(param);
                }
            }

            return new InvocationResult(methodReturn.ReturnValue, outParms.ToArray());
        }

        // Properties
        public TimeSpan ExpirationTime
        {
            get
            {
                return this.expirationTime;
            }
            set
            {
                this.expirationTime = value;
            }
        }

        public int Order
        {
            get
            {
                return this.order;
            }
            set
            {
                this.order = value;
            }
        }
    }
}

