﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity.Utility;
using Microsoft.Practices.Unity.InterceptionExtension;

namespace CachingCallHandler4OutParam
{
    public class InvocationResult
    {
        public object ReturnValue
        { get; private set; }

        public OutputParameter[] Outputs
        { get; set; }

        public InvocationResult(object returnValue, OutputParameter[] outputs)
        {
            Guard.ArgumentNotNull(returnValue, "returnValue");
            this.ReturnValue = returnValue;
            if (null == outputs)
            {
                this.Outputs = new OutputParameter[0];
            }
            else
            {
                this.Outputs = outputs;
            }
        }

        public bool TryGetParameterValue(int index, out object parameterValue)
        {
            parameterValue = null;
            var result = this.Outputs.Where(param => param.Index == index);
            if (result.Count() > 0)
            {
                parameterValue = result.ToArray()[0].Value;
                return true;
            }
            return false;
        }

        public object[] StreamlineArguments(IParameterCollection arguments)
        {
            var list = new List<object>();
            object paramValue;
            for (int i = 0; i < arguments.Count; i++)
            {
                if (this.TryGetParameterValue(i, out paramValue))
                {
                    list.Add(paramValue);
                }
                else
                { 
                    list.Add(arguments[i]);
                }
            }

            return list.ToArray();
        }
    }

    public class OutputParameter
    {
        public object Value
        { get; private set; }

        public int Index
        { get; private set;}

        public OutputParameter(object value, int index)
        {
            this.Value = value;
            this.Index = index;
        }
    }
}



